/**
  *  \file client/act-ship.cc
  *
  *  (c) 2001-2010 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "client/actions.h"
#include "client/scr-sim.h"
#include "ui/msgbox.h"
#include "util/nls.h"
#include "game/sim-game.h"
#include "game/ship.h"
#include "cpluslib/format.h"
#include "game/cmds.h"
#include "game/racename.h"
#include "util/misc.h"
#include "game/planet.h"
#include "ui/util.h"

void
doShipAddToSim(GShip& ship, bool ask)
{
    GSimShip* p = WSimScreen::sim_state.getShipById(ship.getId());
    if (p != 0) {
        // Get permission for overwrite
        if (ask && !UIMessageBox(format(_("The simulation already contains a ship with Id #%d. "
                                          "Replace it?"), ship.getId()),
                                 _("Add to Simulation")).doYesNoDialog())
            return;
    } else {
        // Add new ship
        GSimState::ship_index_t i = WSimScreen::sim_state.addShip();
        p = &WSimScreen::sim_state.getShip(i);
    }

    GSimulatorRealGameInterface::copyShip(*p, ship);
    WSimScreen::sim_state.doUpdates();
}

/** Generic "give unit" interface.
    \param cmds  Command container
    \param cmd   Command to generate
    \param id    Id of unit to process
    \param owner Current owner of unit
    \param title Title of dialog
    \return true if user changed something */
static bool
doGive(GCommandContainer& cmds,
       GCommand::Type cmd,
       int id,
       int owner,
       const char* title)
{
    /* Generate list */
    UIStringList list;
    for (int i = 1; i <= NUM_PLAYERS; ++i) {
        if (i == owner)
            list.add(i, format(_("%X - (don't give it away)"), i));
        else
            list.add(i, format("%X - %s", i, player_racenames.getShortName(i)));
    }

    /* Generate widget */
    UIStandardListbox listbox(0, NUM_PLAYERS, getTextWidth(*fonts[FONT_NORMAL], list) + 20, list);
    listbox.scrollTo(owner-1, 1);
    if (const GCommand* c = cmds.getCommand(cmd, id)) {
        int val;
        if (stringValue(c->getArg(), val))
            listbox.scrollTo(val-1, 1);
    }

    /* Dialog */
    if (listbox.doStandardListbox(title)) {
        int new_owner = listbox.getKey();
        if (new_owner == owner) {
            cmds.removeCommand(cmd, id);
        } else {
            cmds.setCommand(cmd, id, format("%d", new_owner));
        }
        return true;
    } else {
        return false;
    }
}

void
doGiveShip(GShip& ship, GCommandContainer& cmds)
{
    if (doGive(cmds, GCommand::phc_GiveShip, ship.getId(), ship.getOwner(), _("Give Ship To..."))) {
        ship.markDirty();
    }
}

void
doGivePlanet(GPlanet& planet, GCommandContainer& cmds)
{
    if (doGive(cmds, GCommand::phc_GivePlanet, planet.getId(), planet.getOwner(), _("Give Planet To..."))) {
        planet.markDirty();
    }
}
