#
#  Installer for PCC II
#
#  Original version by Christian Brckner
#  This version by Stefan Reuther
#  I compile this with NSIS 2.21
#
#  Instead of using wildcards for installation, recursive delete for removal,
#  this version expects a list of files prepared by prepare2.pl (PCC-files.nsh).
#  It will only install/delete the files explicitly named there. In particular,
#  when the user created own files, those will be kept upon uninstallation.
#  prepare2.pl will also generate the version stamps.
#
#  This now also includes language support for the installer. To translate the
#  installer into your own language, do the following:
#  - copy PCC2-readme.rtf into PCC2-readme-tlh.rtf (where tlh is the suffix for
#    your language), and translate it. Preferrably use a plain text editor, not
#    a word processor, to avoid cluttering it up with word processor control
#    statement crap.
#  - copy PCC2-installer-en.nsh into PCC2-installer-tlh.nsh and translate it.
#    . the LoadLanguageFile statement needs to include your language's NLF
#      file, i.e. Klingon.nlf.
#    . the new readme file name goes into the LicenseLangString statement.
#    . translate the strings in the LangString lines, and respective last
#      parameter in the VIAddVersionKey lines
#    . replace all occurences of LANG_ENGLISH by your language, i.e.
#      LANG_KLINGON.
#  - add an appropriate !include below the other PCC2-installer-xx.nsh includes
#    (English must remain first).
#  Don't forget to send me your translation for inclusion in the release.
#
#  Language support for PCC2 is independant from language support in the
#  installer.
#
#  Note on Registry usage: this places some registry keys under
#  HKLM\Software\Phost. This has to historical reasons, Christian's original
#  installer used that path, and we continue using it to avoid cluttering up
#  the registry. PCC2 still is not "the official PHost client" or somesuch.
#  That aside, I would always spell "PHost" with a capital "H" :-)
#

!verbose 2      # we don't need the script trace

SetCompressor   /SOLID lzma

!include "FileFunc.nsh"
!include "PCC2-files.nsh"
!insertmacro GetDrives
!insertmacro GetSize
!insertmacro GetTime
!insertmacro un.GetSize

!include "version.nsh"
!include "PCC2-Installer-en.nsh"
!include "PCC2-Installer-de.nsh"

Name            "PCC2"
OutFile         "PCC2-Installer.exe"
Caption         "PCC2"
InstallDir      "$PROGRAMFILES\PCC2\"

VIProductVersion "0.${TIMESTAMP_YEAR}.${TIMESTAMP_MONTH}.${TIMESTAMP_DAY}"

XPStyle         on
InstProgressFlags smooth
InstallDirRegKey HKEY_LOCAL_MACHINE "Software\Phost\PCC2" ""

Var     winplandir
Var     winplandirUnchecked
Var     mytemp

##
##  Pages for Installer
##
LicenseBkColor /windows
PageEx license
  Caption $(s_LicenseCaption)
  LicenseText $(s_LicenseText) $(^NextBtn)
  LicenseData $(s_LicenseData)
PageExEnd

PageEx components
  PageCallbacks componentsInit "" ""
PageExEnd

PageEx directory
  # This page prompts the user for the target directory
PageExEnd

PageEx directory
  DirText $(s_WinplanDir)
  DirVar $winplandirUnchecked
  DirVerify leave
  PageCallbacks winplanPre "" winplanPost
PageExEnd

InstallColors /windows
PageEx instfiles
PageExEnd


##
##  Pages for Uninstaller
##
PageEx un.uninstConfirm
  UninstallText $(s_UninstConfirm)
PageExEnd

PageEx un.instfiles
PageExEnd


##
##  Sections
##
#
#  Remember to update componentsInit when adding a section.
#
Section         $(s_SecBaseFiles)
  SetOutPath        $INSTDIR
  !insertmacro      InstallBaseFiles
  !insertmacro      InstallDLLs
  Call              writeResourceConfig
  Call              writeWinplanIntegration
  WriteUninstaller $INSTDIR\uninstaller.exe

  # Registry stuff
  WriteRegStr HKEY_LOCAL_MACHINE "Software\Phost\PCC2" "" "$INSTDIR"
  WriteRegStr HKEY_LOCAL_MACHINE "Software\Phost\PCC2" "winplandir" "$winplandir"

  StrCpy $0 "Software\Microsoft\Windows\CurrentVersion\Uninstall\PCC2"
  WriteRegStr HKEY_LOCAL_MACHINE $0 "DisplayName" $(s_UninstTitle)
  WriteRegStr HKEY_LOCAL_MACHINE $0 "UninstallString" '"$INSTDIR\uninstaller.exe"'
  WriteRegStr HKEY_LOCAL_MACHINE $0 "DisplayIcon" "$INSTDIR\resource\playvcr.bmp"
  WriteRegStr HKEY_LOCAL_MACHINE $0 "InstallLocation" "$INSTDIR"
  WriteRegStr HKEY_LOCAL_MACHINE $0 "Publisher" "Stefan Reuther"
  WriteRegStr HKEY_LOCAL_MACHINE $0 "HelpLink" "http://phost.de/~stefan/pcc2.html"
  WriteRegStr HKEY_LOCAL_MACHINE $0 "URLInfoAbout" "http://phost.de/~stefan/pcc2.html"
  WriteRegStr HKEY_LOCAL_MACHINE $0 "DisplayVersion" "${TIMESTAMP}"
  WriteRegDWORD HKEY_LOCAL_MACHINE $0 "NoModify" 1
  WriteRegDWORD HKEY_LOCAL_MACHINE $0 "NoRepair" 1
SectionEnd

Section         $(s_SecPCC2)
  SetOutPath        $INSTDIR
  !insertmacro      InstallClient
SectionEnd

Section         $(s_SecPlayVCR)
  SetOutPath        $INSTDIR
  !insertmacro      InstallPlayVCR

  # File Type
  WriteRegStr HKEY_CLASSES_ROOT ".ccb" "" "PlayVCR.Simulation"

  WriteRegStr HKEY_CLASSES_ROOT "PCC2.Simulation" "" $(s_CCBAssocName)
  WriteRegStr HKEY_CLASSES_ROOT "PCC2.Simulation\shell\open\command" "" '"$INSTDIR\playvcr.exe" %1'
SectionEnd

Section         $(s_SecUnTRN)
  SetOutPath       $INSTDIR
  !insertmacro     InstallUnTRN
SectionEnd

Section         $(s_SecCommandLine)
  SetOutPath       $INSTDIR
  !insertmacro     InstallCommandLine
SectionEnd

Section         $(s_SecLanguageSupport)
  SetOutPath       $INSTDIR
  !insertmacro     InstallLanguage
SectionEnd

Section         $(s_SecStartMenu)
  AddSize 1

  # This SetOutPath sets the working directory stored in the shortcuts
  SetOutPath        $INSTDIR
  CreateDirectory   "$SMPROGRAMS\PCC2"

  # Parameter for programs
  StrCpy $0 ""
  StrCmp $winplandir "" noArgs
  StrCpy $0 '-dir "$winplandir"'
 noArgs:

  # PCC2 shortcut?
  SectionGetFlags   1 $1
  IntOp  $1 $1 & 1
  IntCmp $1 0 noPCC2Shortcut
  StrCmp $winplandir "" noPCC2WorkDir
  SetOutPath        $winplandir
 noPCC2WorkDir:
  CreateShortCut    "$SMPROGRAMS\PCC2\$(s_SMPCC2Link).lnk" "$INSTDIR\pcc-v2.exe" "" "$INSTDIR\resource\playvcr.bmp" "" "" "" $(s_SMPCC2Comment)
  SetOutPath        $INSTDIR
 noPCC2shortcut:

  # PlayVCR shortcut?
  SectionGetFlags   2 $1
  IntOp  $1 $1 & 1
  IntCmp $1 0 noPlayVCRShortcut
  CreateShortCut    "$SMPROGRAMS\PCC2\$(s_SMPlayVCRLink).lnk" "$INSTDIR\playvcr.exe" $0 "$INSTDIR\resource\playvcr.bmp" "" "" "" $(s_SMPlayVCRComment)
 noPlayVCRShortcut:

  CreateShortCut    "$SMPROGRAMS\PCC2\$(s_SMReadmeLink).lnk" "$INSTDIR\doc\README.doc" "" "" "" "" "" $(s_SMReadmeComment)

  CreateShortCut    "$SMPROGRAMS\PCC2\$(s_SMDocLink).lnk" "$INSTDIR\doc" "" "" "" "" "" $(s_SMDocComment)

  CreateShortCut    "$SMPROGRAMS\PCC2\$(s_SMHelpLink).lnk" "$INSTDIR\doc\pcc2-overview.html" "" "" "" "" "" $(s_SMHelpComment)

  CreateShortCut    "$SMPROGRAMS\PCC2\$(s_SMUninstallLink).lnk" "$INSTDIR\uninstaller.exe" "" "" "" "" "" $(s_SMUninstallComment)
SectionEnd

Section          $(s_SecMapFiles)
  SetOutPath       $INSTDIR
  !insertmacro     InstallMapFiles
SectionEnd


##
##  Uninstaller logic
##
Section         Uninstall
  # First, uninstall Winplan integration
  ClearErrors
  ReadRegStr $winplandir HKEY_LOCAL_MACHINE "Software\Phost\PCC2" "winplandir"
  IfErrors skipUninstallIntegrator
  StrCmp $winplandir "" skipUninstallIntegrator

  # Is winplan.exe actually our program?
  IfFileExists $winplandir\winplano.exe "" skipUninstallIntegrator
  IfFileExists $winplandir\winplan.exe "" skipUninstallIntegrator
  ${un.GetSize} $winplandir "/M=winplan.exe /S=0B /G=0" $0 $1 $2
  IntCmp $0 500000 isWinplan isIntegrator isWinplan
 isIntegrator:
  Delete /REBOOTOK $winplandir\winplan.exe
  Rename /REBOOTOK $winplandir\winplano.exe $winplandir\winplan.exe
  Delete $winplandir\vcr.cfg

 isWinplan:
 skipUninstallIntegrator:
  ClearErrors

  !insertmacro  un.InstallAll
  Delete        "$INSTDIR\resource\cc-res.cfg"
  !insertmacro  un.RemoveDirs
  Delete        "$INSTDIR\uninstaller.exe"
  RMDir         "$INSTDIR"

  IfErrors "" noErrorOnDelete
  MessageBox MB_ICONSTOP|MB_YESNO "$(s_UninstDelete1)$INSTDIR$(s_UninstDelete2)" IDNO noForceDelete
  ClearErrors
  RMDir /r /REBOOTOK "$INSTDIR"

 noForceDelete:
 noErrorOnDelete:

  DeleteRegKey HKEY_LOCAL_MACHINE "SOFTWARE\Phost\PCC2"
  DeleteRegKey HKEY_LOCAL_MACHINE "Software\Microsoft\Windows\CurrentVersion\Uninstall\PCC2"

  # File Type
  DeleteRegKey HKEY_CLASSES_ROOT "PCC2.Simulation"
  ReadRegStr $0 HKEY_CLASSES_ROOT ".ccb" ""
  StrCmp $0 "PlayVCR.Simulation" "" skipUninstallAssociation
  DeleteRegKey HKEY_CLASSES_ROOT ".ccb"
 skipUninstallAssociation:

  # Start Menu
  RMDir /r      "$SMPROGRAMS\PCC2"
SectionEnd


##
##  Functions
##

# Create 'cc-res.cfg'
Function writeResourceConfig
  StrCpy $0 ""
  StrCmp $winplandir "" noWinplanDir

  IfFileExists $winplandir\bmp "" noBmp
  StrCpy $0 "$0wp:$winplandir\bmp$\r$\n"
 noBmp:

  IfFileExists $winplandir\wpvcr.dll "" noWPVCR
  StrCpy $0 "$0wpvcr:$winplandir\wpvcr.dll$\r$\n"
 noWPVCR:

  IfFileExists $winplandir\cc256.res "" noCC256
  StrCpy $0 "$0res:$winplandir\cc256.res$\r$\n"
 noCC256:
 noWinplanDir:

  StrCmp $0 "" noCCRES
  FileOpen $1 "$INSTDIR\resource\cc-res.cfg" "w"
  DetailPrint "$(s_Writing) $INSTDIR\resource\cc-res.cfg..."
  ${GetTime} "" "L" $2 $3 $4 $5 $6 $7 $8
  FileWrite $1 "; PCC Resource Configuration$\r$\n;   generated by installer on $4$3$2, $6:$7:$8$\r$\n$0"
  FileClose $1
 noCCRES:
FunctionEnd

# Write Winplan integrator
Function writeWinplanIntegration
  StrCmp $winplandir "" noWinplan
  IfFileExists $winplandir\winplan.exe "" noWinplan

  # No integrator if we don't install PlayVCR
  SectionGetFlags   2 $1
  IntOp  $1 $1 & 1
  IntCmp $1 0 noWinplan

  # Check whether there already is a Winplan Integrator installed, to
  # avoid that we overwrite a user's real Winplan
  ${GetSize} $winplandir "/M=winplan.exe /S=0B /G=0" $0 $1 $2
  IntCmp $0 500000 isWinplan isIntegrator isWinplan
 isIntegrator:
  DetailPrint $(s_OldIntegrator)
  Goto installIntegrator
 isWinplan:
  DetailPrint $(s_NewIntegrator)
  Delete $winplandir\winplano.exe
  Rename $winplandir\winplan.exe $winplandir\winplano.exe
 installIntegrator:
  File "/oname=$winplandir\winplan.exe" vcrwrap.exe
  FileOpen $1 "$winplandir\vcr.cfg" "w"
  DetailPrint "$(s_Writing) $winplandir\vcr.cfg..."
  FileWrite $1 "playvcr=$INSTDIR\playvcr.exe"
  FileClose $1

 noWinplan:
FunctionEnd


# Look for Winplan, recursively.
# in:  $winplandir -- directory to scan
# out: $winplandir -- "" if Winplan not found, Winplan directory otherwise
Function lookForWinplan
  IfFileExists $winplandir\winplan.exe found

  # We try all combinations of "games", "plan1", "spiele", "planets", and "winplan"
  # that exist on the user's drive.
  IfFileExists $winplandir\games "" notGames
  Push $winplandir
  StrCpy $winplandir $winplandir\games
  Call lookForWinplan
  StrCmp $winplandir "" "" foundPop
  Pop $winplandir

 notGames:
  IfFileExists $winplandir\plan1 "" notPlan1
  Push $winplandir
  StrCpy $winplandir $winplandir\plan1
  Call lookForWinplan
  StrCmp $winplandir "" "" foundPop
  Pop $winplandir

 notPlan1:
  IfFileExists $winplandir\spiele "" notSpiele
  Push $winplandir
  StrCpy $winplandir $winplandir\spiele
  Call lookForWinplan
  StrCmp $winplandir "" "" foundPop
  Pop $winplandir

 notSpiele:
  IfFileExists $winplandir\planets "" notPlanets
  Push $winplandir
  StrCpy $winplandir $winplandir\planets
  Call lookForWinplan
  StrCmp $winplandir "" "" foundPop
  Pop $winplandir

 notPlanets:
  IfFileExists $winplandir\winplan "" notWinplan
  Push $winplandir
  StrCpy $winplandir $winplandir\winplan
  Call lookForWinplan
  StrCmp $winplandir "" "" foundPop
  Pop $winplandir

 notWinplan:
  StrCpy $winplandir ""
  Goto found

 foundPop:
  Pop $mytemp
 found:
FunctionEnd

# Find Winplan on a drive. Used as callback by $GetDrives
# in: $9 -- drive letter ("c:\"), $8 -- drive type          #"
Function lookForWinplanOnDrive
  StrCpy $winplandir "$9"

  # remove trailing \ if there is one:
  StrCpy $mytemp $winplandir "" -1
  StrCmp $mytemp \ "" notSlash
  StrCpy $winplandir $winplandir -1
 notSlash:

  # look for Winplan in that directory
  Call lookForWinplan
  StrCmp $winplandir "" "" notFound
  Push ""
  Goto return
 notFound:
  Push "StopGetDrives"
 return:
FunctionEnd

# Pre function for Winplan directory page
# (Note that we still look for Winplan even if PlayVCR is not to be installed,
# because we want to use Winplan's bitmaps.)
Function winplanPre
  StrCpy $winplandir ""
  ${GetDrives} "HDD" lookForWinplanOnDrive
  StrCpy $winplandirUnchecked $winplandir
FunctionEnd

# Post function for Winplan directory page
Function winplanPost
  StrCmp $winplandirUnchecked "" found
  IfFileExists $winplandirUnchecked\winplan.exe found
  MessageBox MB_YESNO $(s_WinplanNotFound) \
    IDYES installYes IDNO installNo

 installNo:
  Abort

 installYes:
  StrCpy $winplandir ""
  Goto return
 found:
  StrCpy $winplandir $winplandirUnchecked
 return:
FunctionEnd

Function componentsInit
  SectionSetFlags 0 17 # means BaseFiles not unselectable
  SectionSetFlags 7 0  # means MapFiles unselected by default
FunctionEnd

Function un.onUninstSuccess
  IfRebootFlag "" noReboot
  MessageBox MB_ICONSTOP|MB_YESNO $(s_UninstReboot) IDNO noReboot
  Reboot
 noReboot:
FunctionEnd

!verbose 4          # re-enable verbosity to see compression stats
