/**
  *  \file u/t_game_racename.cc
  */

#include "game/racename.h"

#include "u/t_game.h"
#include "u/files.h"
#include "io/memstream.h"

void
GameRaceNameTestSuite::testRaceNameList()
{
    // Construct a race name list
    MemoryStream ms(string_t(file_racenm, file_racenm_size));
    GRaceNameList rnl(ms);

    // Check it
    TS_ASSERT_EQUALS(rnl.getShortName(1), "The Feds");
    TS_ASSERT_EQUALS(rnl.getFullName(1),  "The Solar Federation");
    TS_ASSERT_EQUALS(rnl.getAdjName(1),   "Fed");

    TS_ASSERT_EQUALS(rnl.getShortName(11), "The Colonies");
    TS_ASSERT_EQUALS(rnl.getFullName(11),  "The Missing Colonies of Man");
    TS_ASSERT_EQUALS(rnl.getAdjName(11),   "Colonial");

    // We cannot sensibly check index 0 and 12, because those are mangled
    // by NLS which is in an indeterminate state here. So we just check
    // that they don't throw or otherwise fail.
    rnl.getShortName(0);
    rnl.getFullName(0);
    rnl.getAdjName(0);
    rnl.getShortName(12);
    rnl.getFullName(12);
    rnl.getAdjName(12);

    // Expansions
    TS_ASSERT_EQUALS(rnl.expandNames("a %-5 ship"), "a Privateer ship");
    TS_ASSERT_EQUALS(rnl.expandNames("attack %5!"), "attack The Privateers!");
    TS_ASSERT_EQUALS(rnl.expandNames("%1..."), "The Feds...");
    TS_ASSERT_EQUALS(rnl.expandNames("%1."), "The Feds.");
    TS_ASSERT_EQUALS(rnl.expandNames("%1"), "The Feds");
    TS_ASSERT_EQUALS(rnl.expandNames("%a..."), "The Rebels...");
    TS_ASSERT_EQUALS(rnl.expandNames("...%b"), "...The Colonies");
    TS_ASSERT_EQUALS(rnl.expandNames("%A..."), "The Rebels...");
    TS_ASSERT_EQUALS(rnl.expandNames("...%B"), "...The Colonies");
    TS_ASSERT_EQUALS(rnl.expandNames("%-A..."), "Rebel...");
    TS_ASSERT_EQUALS(rnl.expandNames("...%-B"), "...Colonial");
    TS_ASSERT_EQUALS(rnl.expandNames("%1%2"), "The FedsThe Lizards");
    TS_ASSERT_EQUALS(rnl.expandNames("a%%b"), "a%b");
    TS_ASSERT_EQUALS(rnl.expandNames("%%"), "%");
    TS_ASSERT_EQUALS(rnl.expandNames("%%1"), "%1");

    // I'm not entirely sure about these. They are out of range. Right now,
    // they yield a blank result, but a non-blank result would be well within
    // the specification (such as a rule that '%' quotes the next character).
    TS_ASSERT_EQUALS(rnl.expandNames("%0"), "");
    TS_ASSERT_EQUALS(rnl.expandNames("%01"), "1");
    TS_ASSERT_EQUALS(rnl.expandNames("%c"), "");
    TS_ASSERT_EQUALS(rnl.expandNames("%x"), "");
    TS_ASSERT_EQUALS(rnl.expandNames("%."), "");
    TS_ASSERT_EQUALS(rnl.expandNames("%-"), "");
    TS_ASSERT_EQUALS(rnl.expandNames("%-."), "");
    TS_ASSERT_EQUALS(rnl.expandNames("%"), "");
}
